# R script for the assignment on co-evolution of alcohol consumption
# & friendship


# Thanks to Andrea Knecht who collected the data
# Thanks to Christian Steglich who taught us (lazy students) SIENA  (as far as possible)

# load RSiena commands:
install.packages("RSiena", repos="http://R-Forge.R-project.org", dependencies = TRUE)
library(RSiena)

# set working directory to where the data are:
# setwd("<type your directory name of choice here>")   

# set working directory to where the data are:

setwd("C:/temp/") 


# list.files()


# Read data sets:
# net1 <- s501 Datensaetze liegen intern auch vor
#net1 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/s50-network1.dat"))
#net2 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/s50-network2.dat"))
#net3 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/s50-network3.dat"))
#alcohol <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/s50-alcohol.dat"))


net1 <- as.matrix(read.table("s50-network1.dat"))
net2 <- as.matrix(read.table("s50-network2.dat"))
net3 <- as.matrix(read.table("s50-network3.dat"))
alcohol <- as.matrix(read.table("s50-alcohol.dat"))

alcohol
# No need to recode data or identify missings (this data set does not
# contain any missing data).

# Identify dependent network variable:
friendship <- sienaNet(array(c(net1, net2, net3), dim=c(50, 50, 3)))

# Identify dependent behavior variable:
drinking <- sienaNet(alcohol, type="behavior") # type="behavior" is important 
                                               # it makes the model to analyse 
                                               # coevolution

# Bind data together for Siena analysis:
CoEvolutionData <- sienaDataCreate(friendship,drinking) # bind together both objects
                                                        # into one data object
                                                        # the same logic applies to  multiplex networks

# Create effects object for model specification:
CoEvolutionEffects <- getEffects(CoEvolutionData)
CoEvolutionEffects 

# Write first descriptive results to protocol file (optional):
print01Report(CoEvolutionData, modelname = 'CoEvol-init')

# kreuztabelle der uebergaenge
table(alcohol[,1],alcohol[,2])

fix(CoEvolutionEffects) # at the bottom of the table we now have 
                        #  new variables / effects for  "drinking"
# hypotheses 1: reziprocal relationships recipDeg
# Specify the model according to the assignment.

# Hypothesis a); expected sign: + for isolate effect
# Hypothesis a); expected sign: - # name: erste Spalte der Effekttabelle, dort muss man den Namen der AV angeben
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,recipDeg, interaction1="friendship",name="drinking")
# Hypothesis b); expected sign: +
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,egoX,interaction1="drinking",name="friendship")
# Hypothesis c); expected sign: +
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,simX,interaction1="drinking",name="friendship")
# Hypothesis d); expected sign: -
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,altX,interaction1="drinking",name="friendship")
# Hypothesis e); expected sign: +
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,totSim, interaction1="friendship",name="drinking")
# total similarity effect: "totSim" // average similarity: "avSim" // average  alter effect: "avAlt"

# Add structural effects: wichtig hier die Netzwerkstrukturen, sonst koennten andere Selektionsvariablen falsch geschaetzt
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,recipDeg,transTrip,cycle3,name="friendship")
# behavior drinking reciprocated degree: "recipDeg"
CoEvolutionEffects 


# Now create a model object:
CoEvolutionModel <- sienaModelCreate(useStdInits = FALSE, 
  projname = 'CoEvol-results')

# Estimate the model:
CoEvolutionResults <- siena07(CoEvolutionModel, data=CoEvolutionData,
  effects=CoEvolutionEffects, batch=FALSE, verbose=FALSE)

CoEvolutionResults 


## repeat with different starting values : prevAns = CoEvolutionResults => take last estimates as initial values 
## usually, convergence improves (see convergence t-ratios)
CoEvolutionResults1 <- siena07(CoEvolutionModel, data=CoEvolutionData,
                              effects=CoEvolutionEffects, prevAns = CoEvolutionResults,  batch=FALSE, verbose=FALSE)
CoEvolutionResults1 


CoEvolutionResults2 <- siena07(CoEvolutionModel, data=CoEvolutionData,
                               effects=CoEvolutionEffects, prevAns = CoEvolutionResults1,  batch=FALSE, verbose=FALSE)
CoEvolutionResults2 



CoEvolutionEffects

# now remove the totSim effect with "include=FALSE"
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,totSim, include=FALSE, interaction1="friendship",name="drinking")
CoEvolutionEffects

CoEvolutionResults5 <- siena07(CoEvolutionModel, data=CoEvolutionData,
                               effects=CoEvolutionEffects,  batch=FALSE, verbose=FALSE)
CoEvolutionResults5 

# now remove recipDeg, and include again totSIM
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,recipDeg, include=FALSE, interaction1="friendship",name="drinking")
CoEvolutionEffects <- includeEffects(CoEvolutionEffects,totSim, include=TRUE, interaction1="friendship",name="drinking")
CoEvolutionEffects

# now a model with totSim, but not with recipDeg 
CoEvolutionResults6 <- siena07(CoEvolutionModel, data=CoEvolutionData,
                               effects=CoEvolutionEffects,  batch=FALSE, verbose=FALSE)
CoEvolutionResults6 

