

############################
############################
## vectors and matrices ####
############################
############################

# see: https://www.youtube.com/watch?v=nnx8_2Ckt-g
# see: https://www.youtube.com/watch?v=h6dbGAvZoL0

v <- c(5.5,2.3, 1, 1, 0.8)

v
v[1] # extract and show 1st elment 
v[1:4]

mat <- matrix(1:12, nrow = 4)
mat

# extract element from matrix: refer to row and column
mat[2,2]
mat[,2]
mat[2,]


# combine with relational operators
v
v[v > 1]   # display elements
v > 1      # display TRUE or FALSE

# display 3rd element
x <- c(1.1, 8.2, 2.3, 7.4, 3.5, 4.6, 6.6, 5.4)
x[3]
i <- 3
x[i]

x[c(1,3,5)]   # get the respective elements
i <- c(1,3,5)
x[i]

# logical vectors used to extract c(1,3,5)
x[c(TRUE,FALSE,TRUE,FALSE,TRUE,FALSE,FALSE,FALSE)]

i <- c(TRUE,FALSE,TRUE,FALSE,TRUE,FALSE,FALSE,FALSE)
x[i]

# modify values inside a vector
y <- x
y[3] <- 77
y <- x
y[c(1,3,5)] <- 77
y <- x
y[c(1,3,5)] <- c(20,40,60)

y <- x
y>5       # gives only TRUE or FALSE
y[y>5]    # gives the numbers

y[y>5] <- 77

y>5 
which(y>5)

# conditional replace
y <- x
y[which(x>5)] <- 1
y[which(x<=5)] <- 0
y


############################
############################
#### data frames ###########
############################
############################

# https://www.youtube.com/watch?v=Tt4vYYCEzLY

rm(list=ls())  # remove al object from workspace

iris           # example data frame
class(iris)    # what kind of object is it?
str(iris)      # describe the data frame. Here you see the class of the vectors
               # Species is a factor with 3 levels. Factor is a nominal scaled variable
table(iris$Species)
attach(iris)    # switch to the common "data set and its variables" format with "attach". Variables 
                # can be adressed without referencing the object
table(Species)
detach(iris)   # data frame is detached again

# extract 1st rows
iris[1, ]     # 1st row, all columns
iris[1:10, ]  # 1st 10 lines

iris[ , 1]    # all lines, 1st column
iris$Sepal.Length

head(iris) # look at the 1st pat of the data set

# extract specific content, by a vector with "c" (content)
iris[c("Sepal.Length", "Sepal.Width","Petal.Width")] 

# subset of columns
sub_set <- iris[c("Sepal.Length", "Sepal.Width","Petal.Width")] # if you like put this into an object
sub_set
rm(sub_set) # remove the new object

# subset of rows
sub_set <- subset(iris, Sepal.Length > 5)
sub_set

# subset of rows & columns
sub_set <- subset(iris, Sepal.Length > 5, select = c(Petal.Width, Species))
sub_set

# identify large sepal length with 1, else 0
iris$large.sepal <- 0
iris$large.sepal[which(iris$Sepal.Length>5)] <- 1


# select variables only for species "Setosa"
iris[iris$Species == "setosa" , c("Sepal.Length", "Sepal.Width")]

attach(iris) # attach data frame object, so that you don't need to adress the object
iris.mat <- as.matrix(cbind(Sepal.Length, Sepal.Width, Petal.Length, Petal.Width))
str(iris.mat) # compact display of an R object
iris.mat[1:10,] # display 1st 10 rows, all columns
iris.txt <- Species # works, because data frame object is attached
iris.txt
str(iris.txt) # object type is factor with 3 levels (here: 3 categories of spezies)
detach(iris)

iris.all <- cbind(iris.mat, iris.txt) # converts "Species" into numeric by merging factor object 
iris.all

# the most convenient for stata users: data frame onject type
iris.all <- as.data.frame(cbind(iris.mat, as.character(iris$Species))) # cbind of data frame does leave variable as string
iris.all
class(iris.all)

rm(iris.all)
iris.all <- as.matrix(cbind(iris.mat, as.character(iris$Species))) # converts all into character
iris.all
str(iris.all)
class(iris.all)

class(iris)
with(iris, mean(Sepal.Length)) # use "with" to compute mean of vector in data frame
mean(iris$Sepal.Length)
mean(iris$Sepal.Width)



two <- data.frame(cbind(iris$Sepal.Length, iris$Sepal.Width))
class(two)

mean(two$Sepal.Length) # Problem: variable name do not exist anymore
mean(two[ , 1])
mean(two[ , 2])

two <- as.matrix(cbind(iris$Sepal.Length, iris$Sepal.Width))
mean(two[ , 1])
mean(two[ , 2])

two <- subset(iris, select = Sepal.Width) 
two[1:10,]
mean(two)

iris$var <- round(iris$Sepal.Width) - 1 # add new variable to data frame
table(iris$var)
iris$var.factor <- factor(iris$var, levels=c(1,2,3), labels=c("one", "two", "three")) # convert to nominal factor variable
table(iris$var.factor)

levels(iris$var.factor)

iris$var.factor.num <- as.numeric(iris$var.factor)

cor(iris$var.factor.num,iris$var)

install.packages('Hmisc') # try twice, if necessary
library(Hmisc)

label(iris$large.sepal) <- "this is a factor variable"
describe(iris$large.sepal)


# recode, reverse poles
install.packages('car')
library(car)

iris$spec <- as.numeric(iris$Species)

iris$spec.r <- recode(iris$spec, '1=3; 2=2; 3=1;', as.factor.result=FALSE)
cor(iris$spec,iris$spec.r) # correlation is -1


# mean scale over four variables
str(iris)
iris$iris.mean <- (iris$Sepal.Length + iris$Sepal.Width + iris$Petal.Length + iris$Petal.Width)/4
iris[1:5,]

# sort cases in data frame
new.iris <- iris[order(iris$Sepal.Length),]  # ascending
new.iris <- iris[order(-iris$Sepal.Length),] # descending

# add the ratio of two variables "within" the data frame
iris <- within(iris, ratioLW <- Sepal.Length / Sepal.Width)
head(iris)
iris <- subset(iris, select = -ratioLW) # drop variable "ratioLW" from data frame
head(iris)
iris$ratioLW <- iris$Sepal.Length / iris$Sepal.Width # alternative to crate ratioLW
head(iris)

# add a chronological id to data frame
nrow(iris)
iris$id <- c(1:nrow(iris))

# subset for selection of cases

############################
############################
#### for loops   ###########
############################
############################

# print out values of Sepal.Length as a sequence 
for (i in 1:nrow(iris)) {
  p <- iris$Sepal.Length
  print(p[i]) 
}

# compute mean, but skip one case in each step ("jacknife")
test <- c(1,2,3)
for (i in 1:length(test)) {
  print(mean(test[-i]))
}

# compute mean, but skip one case in each step ("jacknife")
for (i in 1:nrow(iris)) {
  p <- iris$Sepal.Length[-i]
  print(mean(p))
}

# compute mean, but skip one case in each step ("jacknife"), even shorter
for (i in 1:nrow(iris)) {
  print(mean(iris$Sepal.Length[-i]))
}

# list objects and for-loops: create several matrices
rm(list=ls())  # remove al object from workspace

mat <- list()
str(mat)
for (i in 1:4) {
   mat[[i]] <- matrix(1:12, nrow = 4)
   diag(mat[[i]]) <- i
   print(mat[[i]])
}
mat[[1]]
mat[[2]]

class(mat)

# replace diagonal in each matrix with 0
for (i in 1:4) {
  mat[[i]] <- matrix(1:12, nrow = 4)
  diag(mat[[i]]) <- 0
  print(mat[[i]])
}
mat[[1]]
mat[[2]]

mat.df <- data.frame(mat)
mat.df

# unlist(mat, recursive = FALSE, use.names = TRUE)

mat2 <- mat[[2]]
mat2


mat_ <- list()
for (i in 1:4) {
  mat_[[i]] <- mat[[i]]
  print(mat_[[i]])
  }


