## R einstellen


install.packages('metafor')
install.packages('statnet')


library(statnet)
library(metafor)


rm(list = ls())

cl.n    <- 1:4
gesch   <- list()
friends <- list()
naehe   <- list()

actor   <- list()


n <- read.table("http://www.barkhof.uni-bremen.de/~mwindzio/naehe1.dat")
n

# achtung, hier mussten die ersten beiden Spalten der Netzwerkmatrizen geloescht werden [, -(1:2)]
# damit die matrix quadratisch ist
for (i in 1:4) {
  
  tempn  <- paste("http://www.barkhof.uni-bremen.de/~mwindzio/geschaefte",i,".dat", sep="")
  tempn_ <- read.table(file=tempn, header = FALSE, na.strings = "99")
  gesch[[i]] <- network(tempn_[, -(1:2)], matrix.type="adjacency", directed=TRUE)
  
  tempn  <- paste("http://www.barkhof.uni-bremen.de/~mwindzio/friends",i,".dat", sep="")
  tempn_ <- read.table(file=tempn, header = FALSE, na.strings = "99")
  friends[[i]] <- network(tempn_[, -(1:2)], matrix.type="adjacency", directed=TRUE)
  
  tempn  <- paste("http://www.barkhof.uni-bremen.de/~mwindzio/naehe",i,".dat", sep="")
  tempn_ <- read.table(file=tempn, header = FALSE, na.strings = "99")
  naehe[[i]] <- network(tempn_[, -(1:2)], matrix.type="adjacency", directed=TRUE)
  
  tempn  <- paste("http://www.barkhof.uni-bremen.de/~mwindzio/actor",i,".dat", sep="")
  actor[[i]] <- read.table(file=tempn, header = TRUE, na.strings = "99")
  
  friends[[i]] %v% "ethnie" <- actor[[i]]$ethnie 
  friends[[i]] %v% "buech" <- actor[[i]]$buech 
  friends[[i]] %v% "wohnform" <- actor[[i]]$wohnform 
  friends[[i]] %v% "k_eigzi" <- actor[[i]]$k_eigzi 
  friends[[i]] %v% "D2kultur" <- actor[[i]]$D2kultur 
  friends[[i]] %v% "D4sk_temper" <- actor[[i]]$D4sk_temper 
  friends[[i]] %v% "D5soz_spass" <- actor[[i]]$D5soz_spass 
  friends[[i]] %v% "D6leist" <- actor[[i]]$D6leist 
  friends[[i]] %v% "leist_mathe" <- actor[[i]]$leist_mathe 
  friends[[i]] %v% "leist_deutsch" <- actor[[i]]$leist_deutsch 
  friends[[i]] %v% "D8mutter_kontr" <- actor[[i]]$D8mutter_kontr 
  friends[[i]] %v% "maed" <- actor[[i]]$maed
  
  friends[[i]] %v% "gesch" <- gesch[[i]]
   
}

friendn_ <- ergm(friends[[1]] ~ edges +  gwesp(decay=0.3, fixed=TRUE) + mutual
                     +nodematch("ethnie") + nodeocov("buech") +nodeocov("D4sk_temper")
                     + absdiff("D6leist", pow=1),
                     verbose=FALSE, control.ergm=c(seed=1234,MCMCsamplesize=1024,MCMLE.maxit=30))

summary(friendn_)

########################
#### friends   ######
########################

rm(coefs, ses, ns, net_no, net)
coefs <- NULL
ses <- NULL
ns <- NULL
net_no <- NULL
net <- NULL

coef<-list()
se<-list()
n<-list()

friendn <- list()
#naehe <- list()
model <-list()

cl.nNEW    <- c(1:4)


for (i in cl.nNEW) {

  friendn[[i]] <- ergm(friends[[i]] ~ edges + gwesp(decay=0.3, fixed=TRUE) + mutual
                          +nodematch("ethnie") + nodeocov("buech") +nodeocov("D4sk_temper")
                          + absdiff("D6leist", pow=1),
                          verbose=FALSE, control.ergm=c(seed=1234,MCMCsamplesize=1024,MCMLE.maxit=30))
  # herausschreiben der Koeffizienten
  coef[[i]] <- friendn[[i]]$coef
  se[[i]] <- summary(friendn[[i]])$asyse 
  n[[i]] <- network.size(friends[[i]])
  net[i] <- i
  
  # umbennen der Variablennamen im Objekt der Standard errors
  names(se[[i]]) <- c("edges_se", "gwesp.fixed.0.3_se", "mutual_se",
                      "nodematch.ethnien_se","nodeocov.buech_se", "nodeocov.D4sk_temper_se", 
                      "absdiff_D6leist_se")     
  coefs <- rbind(coefs,coef[[i]]) # in each step append currents result
  ses <- rbind(ses,se[[i]])
  ns <- rbind(ns,n[[i]])
  net_no <- rbind(net_no, net[i])
  print(net[[i]])
}

results <- cbind(coefs,ses, net_no)
results
# hier evtl.  die Bereinigung um "schlechte" Klassen, dier nicht liefen: 12, 13,28, 31
coef <- results[(net_no == 1 | net_no == 2 | net_no == 3| net_no == 4 ), ]
coef  


##########################
## prepare data for ######
## meta analysis #########
##########################

b   <- coef[ , "edges"]
se <- coef[ , "edges_se"]
dat <- cbind(b, se)

# method="REML"

# meta analysis for "edges" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="edges")  

##############

b   <- coef[ , "mutual"]
se <- coef[ , "mutual_se"]
dat <- cbind(b, se)

# meta analysis for "mutuality" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="mutual")  

##############
  
b   <- coef[ , "gwesp.fixed.0.3"]
se <- coef[ , "gwesp.fixed.0.3_se"]
dat <- cbind(b, se)

# meta analysis for "gwesp" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="gwesp")  
 

##############
b   <- coef[ , "nodematch.ethnie"]
se <- coef[ , "nodematch.ethnien_se"]
dat <- cbind(b, se)

# meta analysis for "nodematch.ethnien" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="nodematch.ethnien")  


##############
b   <- coef[ , "nodeocov.buech"]
se <- coef[ , "nodeocov.buech_se"]
dat <- cbind(b, se)

# meta analysis for "nodeocov.buech" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="nodeocov.buech") 


##############
b   <- coef[ , "nodeocov.D4sk_temper"]
se <- coef[ , "nodeocov.D4sk_temper_se"]
dat <- cbind(b, se)

# meta analysis for "nodeocov.D4sk_temper" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="nodeocov.D4sk_temper") 


##############
b   <- coef[ , "absdiff.D6leist"]
se <- coef[ , "absdiff_D6leist_se"]
dat <- cbind(b, se)

# meta analysis for "absdiff_D6leist" term
res.RE <- rma(b, se, data=dat, method="REML")
res.RE

forest(b, sei=se, xlab="absdiff_D6leist") 
 













