# math_network_convert.R
install.packages('intergraph')
install.packages('statnet')
install.packages('igraph')
install.packages('sna')
install.packages('latticeExtra')
#install.packages('rgl', dependencies=TRUE) # great tools, but too much for today
#library(rgl)



library(network)
library(sna)
library(igraph)
library(intergraph)
# how to enter network data in an efficient way? 
# use edgelists
# But: hwo do edgelist deal with isolates?


# set path to data
#setwd("c:/temp/")# mind the "/"

## ex. 1, convert into data frame object, no chronology assumed
## when entering data: insert self-tie for isolates, see 12 => 12
t <- read.table(file="http://www.barkhof.uni-bremen.de/~mwindzio/test_edge.dat", na.strings="99",  header=FALSE)
t
graph.data.frame(t) # igraph function, defines edgelist
class(graph.data.frame(t))

tt <- graph.data.frame(t) # igraph function, defines edgelist
tt 
tt <- get.adjacency(tt, sparse = FALSE) # creates adjacency >matrix< from edge list CAUTION: isolates do not exist!!
tt # but look at the matrix!! => where is actor 1, 3, 8, 10?




## ex. 2 create chronology by using the graph.edgelist function
t <- read.table(file="http://www.barkhof.uni-bremen.de/~mwindzio/test_edge.dat", na.strings="99",  header=FALSE)
class(get.adjacency(graph.edgelist(as.matrix(t)), sparse = FALSE))

tt <- get.adjacency(graph.edgelist(as.matrix(t)), sparse = FALSE)
tt # now >all< actors exist in matrix, also isolates

# ex. 3 enter all isolates with self-relations. Than replace the main diagonal with zeros 
diag(tt) <- 0 # hier werden alle Diagonalelemente mit null ueberschrieben
tt  

netw <- network(tt) # defined as network object type
netw

plot(netw)
gplot(netw)

# apply igraph function to network object
betweenness(asIgraph(netw), directed = TRUE, weights = NULL,
            nobigint = TRUE, normalized = FALSE)
# betweenness centrality also of edges!
edge_betweenness(asIgraph(netw), directed = TRUE, weights = NULL)

netwIG <- asIgraph(netw) # now converted into igraph onject type
netwIG

# apply igraph functions here
degree(netwIG)
degree(netwIG, mode = c("in"))
degree(netwIG, mode = c("out"))
degree(netwIG, mode = c("total"))

# try to compute graph density with "gden", which is a sna function
gden(netwIG) # error! can't apply sna function to igraph object
gden(asNetwork(netwIG)) # error! can't apply sna function to igraph object


# plotting 
friendS <- read.table(file="http://www.barkhof.uni-bremen.de/~mwindzio/fr3.dat", na.strings="99",  header=FALSE)
attributes <- read.table(file="http://www.barkhof.uni-bremen.de/~mwindzio/attribute3.dat", na.strings="99",  header=TRUE)

# convert matrix into network matrix
friends <- network(friendS,matrix.type="adjacency",directed=TRUE)
friends

friends %v% "alter" <- attributes$f2Alter
friends %v% "frau" <- attributes$frau
friends %v% "migrant" <- attributes$mh
friends

gden(friends) # density from sna
edge_density(asIgraph(friends), loops = FALSE) # density from igraph

plot(friends)
gplot(friends)

# nice plotting with igraph
# also: how to save graphs graph_from_adjacency_matrix
l <- igraph::layout.kamada.kawai(   igraph::graph_from_adjacency_matrix(as.matrix(friends), mode = c("directed")  )   )

igraph.options(vertex.label=NA, vertex.size=8, vertex.color="red" , edge.arrow.size=0.1, edge.color="black")
plot(igraph::graph_from_adjacency_matrix(as.matrix(friends), mode = c("directed")  ), layout = l)

igraph.options(vertex.label=NA, vertex.size=8, vertex.color="gray" , edge.arrow.size=0.1, edge.color="black")
plot(igraph::graph_from_adjacency_matrix(as.matrix(friends), mode = c("directed")  ), layout = l)

igraph.options(vertex.label=NA, vertex.size=8, vertex.color="gray" , edge.arrow.size=0.05, edge.color="black")
plot(igraph::graph_from_adjacency_matrix(as.matrix(friends), mode = c("directed")  ), layout = l)

# now create permanent igraph object
net_to_plot <- asIgraph(friends)
net_to_plot
plot(net_to_plot)

# show edges and vertices separately
E(net_to_plot)
V(net_to_plot)

# attention: attribute names come now from the network type object, as we have defined above
V(net_to_plot)$migrant
V(net_to_plot)[migrant==1]

# get id numbers
id <- 1:nrow(attributes) # the same as 1:109
id

# add attribute to igraph object
V(net_to_plot)$id <- id
net_to_plot

plot(net_to_plot, vertex.label=id)
plot(net_to_plot, vertex.label=id, vertex.size=12.5)
plot(net_to_plot, vertex.label=id, vertex.size=10,  edge.curved=0.4) # vertext.label.cex=2,

# compute indegree, add to object and use to plot
deg <- degree(net_to_plot, mode="all")
V(net_to_plot)$vsize <- log(deg+0.1)*3

plot(net_to_plot,  vertex.size=V(net_to_plot)$vsize) # vertex.label=id, vertext.label.cex=0.5

migColor<-ifelse(attributes$mh==1,yes="orange",no="gray60")
girl_shape <-ifelse(attributes$frau==1,yes="circle",no="square")

V(net_to_plot)$shape <- girl_shape
V(net_to_plot)$color <- migColor
plot(net_to_plot, vertex.size=V(net_to_plot)$vsize, 
          vertex.shape=V(net_to_plot)$shape, vertex.color=V(net_to_plot)$color)

plot(net_to_plot, vertex.size=V(net_to_plot)$vsize, 
     vertex.shape=V(net_to_plot)$shape, vertex.color=V(net_to_plot)$color,
     vertex.label=id, vertext.label.cex=0.8)


win.metafile("c:/temp/file.wmf")
plot(net_to_plot, vertex.size=V(net_to_plot)$vsize, 
     vertex.shape=V(net_to_plot)$shape, vertex.color=V(net_to_plot)$color, 
     layout = l)
legend(x=-1, y=-0.5, c("no migrant", "migrant" ), 
       c("gray60","orange"),pt.bg=c("gray60","orange"),box.lty=0, pch=21, bty="n")
dev.off()

pdf("c:/temp/file.pdf")
plot(net_to_plot, vertex.size=V(net_to_plot)$vsize, 
     vertex.shape=V(net_to_plot)$shape, vertex.color=V(net_to_plot)$color, 
     layout = l)
legend(x=-1, y=-0.5, c("no migrant", "migrant" ), 
       c("gray60","orange"),pt.bg=c("gray60","orange"),box.lty=0, pch=21, bty="n")
dev.off()



