######################################################
# R script for assignment on segregation / homophily #
######################################################

# Thanks to Andrea Knecht who collected the data
# Thanks to Christian Steglich who taught us (lazy students) SIENA  (as far as possible)

# load RSiena commands:
install.packages('statnet')
# statnet::update_statnet()
# load RSiena commands:
install.packages("RSiena", dependencies = TRUE)
library(RSiena)
library(statnet)


# set path to data
# setwd("E:/lehre/aktuell/sose2016/Netzwerkanalyse/data/") # mind the "/"
#setwd("c:/temp/") 

# list.files()

# read the data sets for the assignment:
friendship.net1 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/klas03e-friends-waveA.dat",na.strings="9")) # read in matrices for each t, missings are "9"
friendship.net2 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/klas03e-friends-waveB.dat",na.strings="9"))
friendship.net3 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/klas03e-friends-waveC.dat",na.strings="9"))
friendship.net4 <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/klas03e-friends-waveD.dat",na.strings="9"))
primary.net <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/klas03e-primary.dat",na.strings="9")) # dyadic covariate: were they friends already in elementary school?

demographics <- as.matrix(read.table("http://www.barkhof.uni-bremen.de/~mwindzio/klas03e-demographics.dat",na.strings="0"))  # "demographic" actor attributes, last column is religion of father

#friendship.net1 <- as.matrix(read.table("klas03e-friends-waveA.dat",na.strings="9")) # read in matrices for each t, missings are "9"
#friendship.net2 <- as.matrix(read.table("klas03e-friends-waveB.dat",na.strings="9"))
#friendship.net3 <- as.matrix(read.table("klas03e-friends-waveC.dat",na.strings="9"))
#friendship.net4 <- as.matrix(read.table("klas03e-friends-waveD.dat",na.strings="9"))
#primary.net <- as.matrix(read.table("klas03e-primary.dat",na.strings="9")) # dyadic covariate: were they friends already in elementary school?

#demographics <- as.matrix(read.table("klas03e-demographics.dat",na.strings="0"))  # "demographic" actor attributes, last column is religion of father

                                                                                  # 1st column is gender, see .csv file
demographics # 1st column is gender

f1 <- network(friendship.net1)
f2 <- network(friendship.net2)
f3 <- network(friendship.net3)
f4 <- network(friendship.net4)

plot(f1)
plot(f2)
plot(f3)
plot(f4)

# plot 4 waves into one graphic
# op <- par( mfcol=c(1,2), mfrow=c(1,2))
par(mar = rep(2, 4))
par(mfrow=c(2,2))
plot(f1)
plot(f2)
plot(f3)
plot(f4)
# The "na.string" option identifies missing data codes.
# Actually, class 03e does not contain much missing data. This is different
# for other classroom data from Knecht's study, which we will be addressing
# in other exercises.



# find out number of actors in data sets:
numberActors <- dim(demographics)[1] # compute number of actors and write into object 
numberActors # show result
nrow(demographics) # the same: how many rows (actors) does the data set include?

# identify dependent variable for the analysis, put in an object, no. of actors is important information, 25x25 and 4 measurement occasions:
friendship <- sienaNet(array(c(friendship.net1,friendship.net2,friendship.net3,friendship.net4),dim=c(numberActors,numberActors,4))) # die vier verschiedenen Messzeitpunkte werden zusammengeschrieben in array
# now we have a SIENA specific data object, network is the outcome

# identify covariates for the analysis:
sex.M <- coCovar(demographics[,1]) # time-invarying covariate (all rows and column 1 out of object "demographics"(gender) ) 
                                   # is written into Siena object. "male" is the higher value
fath_rel <- coCovar(demographics[,4]) # father's religion from column 4

primary <- coDyadCovar(primary.net) # time-invarying covariate at the dyad-level (matrix) is written into Siena object 



# combine data for the analysis:
segregationdata <- sienaDataCreate(friendship,sex.M,fath_rel,primary) # combine all three data sets in joint Siena Object. Outcome and three covariates 
# we do  not need any other object anymore 

# "get effects" table for model specification:
segregationeffects <- getEffects(segregationdata) # here we can select the network structure effects we need  
segregationdata
segregationeffects

# generate initial descriptive outputfile. Here also the Jaccard-Index is reportet
# this is written into a file called "segregation.descriptive.out"
print01Report(segregationdata,
  modelname='segregation.descriptive') # check der Daten, kann Hinweise auf Probleme sein, parsing, Deskription fuer jeden Akteur


# !! NOTE TO MAC USERS: DON'T RUN THE FOLLOWING COMMAND    !!
# !! IT CAN CORRUPT THE EFFECTS OBJECT (UNLESS IN RSTUDIO) !!
# which effects are available?

# Siena opens another window where we find all possible effects
# These are all structural effects, but also covariate effects - as specified
# in the object "segregationeffects"
fix(segregationeffects)
# Take a thorough look at the 'shortName' column, as it contains the
# names by which you can identify effects in a script!
# ((please close the screen with the effects object again))

# which effects are already chosen?
segregationeffects
# rate parameters, outdegree and reciprocity are included by default.


# now select effects for the first model accordign to assignment...
# effects that should be estimated can be in included or excluded.
# this applies to both actor attribute/ dyadic effects as well as "structural" effects
# so >here< (below) the model specification happens

# >> first exclude 'reciprocity':
segregationeffects <- setEffect(segregationeffects,recip,type='eval',include=FALSE)
# >> then include the effect of primary school friendship:
segregationeffects <- includeEffects(segregationeffects,X,interaction1='primary')
# >> then include the religion-related effects:
segregationeffects <- includeEffects(segregationeffects,sameX,interaction1='fath_rel') # sameX
# >> finally the sex-related effects:
segregationeffects <- includeEffects(segregationeffects,egoX,altX,sameX,interaction1='sex.M') # egoX and alterX
# the selection can also be done interactively by calling fix(segregationeffects)
# egoX, altX, sameX => X value of ego, X value of alter, effect of same characteristic
# here, "interaction1" means how the covariate is related to having a tie


# check if specification of effects is correct
segregationeffects


# create first model for estimation: here we estimate the fist model  
first.model <- sienaModelCreate(useStdInits=FALSE,projname='segregation.first',
  cond=FALSE)
# the last option 'cond=FALSE' switches to unconditional moment estimation,
# i.e., lets rate parameters be estimated instead of conditioning on the
# observed number of changes between networks."useStdInits=FALSE" => Where do initial values for the algorith come from? 
# initial values in the effects object are used. "cond=False" => conditional or unconditional simulation (simulation conditional)
# on specific covariates

# do you still remember which effects you have specified? No? So have a look then: 
segregationeffects


# estimate first model:
first.results <- siena07(first.model,data=segregationdata, 
  effects=segregationeffects,batch=FALSE,verbose=FALSE)
# Results are written to file "segregation.first.out" in the working directory.
# The main ingredients can be seen by calling the results object:
first.results # here are the results

# If estimation results should not show sufficient convergence, another run
# with better starting values may help. The option 'prevAns' makes it possible
# to use the last available estimates as starting values:
first.results <- siena07(first.model,data=segregationdata, 
  effects=segregationeffects,batch=FALSE,verbose=FALSE,prevAns=first.results)
first.results

# Convergence statistics should have improved. Note that new results will be
# appended to the existing file "segregation.first.out", so scroll down for latest
# estimates.

# Now re-include reciprocity effect for second model:
segregationeffects <- includeEffects(segregationeffects,recip)
segregationeffects

# create second model for estimation:
second.model <- sienaModelCreate(useStdInits=FALSE,projname='segregation.second',
  cond=FALSE)

# estimate second model:
second.results <- siena07(second.model,data=segregationdata, 
  effects=segregationeffects,batch=FALSE,verbose=FALSE)
# Results are written to file "segregation.second.out" in the working directory.
second.results

# again use second results as starting values for next model run
second.results <- siena07(second.model,data=segregationdata, 
                          effects=segregationeffects,batch=FALSE,verbose=FALSE,prevAns=second.results)
second.results

# include transitivity effect for third model:
segregationeffects <- includeEffects(segregationeffects,transTrip)
segregationeffects

# create third model for estimation:
third.model <- sienaModelCreate(useStdInits=FALSE,projname='segregation.third',
  cond=FALSE)
third.model
# estimate third model:
third.results <- siena07(third.model,data=segregationdata, 
  effects=segregationeffects,batch=FALSE,verbose=FALSE)
# Results are written to file "segregation.third.out" in the working directory.
third.results

# again use third results as starting values for next model run, try to improve convergence
third.results <- siena07(third.model,data=segregationdata, 
                         effects=segregationeffects,batch=FALSE,verbose=FALSE,prevAns=third.results, returnDeps=TRUE)
third.results

############################################################################################
## Interpretation of effects: actor behaves 'as if' he/she had preference for _effect_ #####
## This applies to the evaluation function. But Effect can be also estimated in the ########
## "creation" function: type='creation' ####################################################
############################################################################################

# religion of father: drop effect on eval (default)
segregationeffects <- includeEffects(segregationeffects,sameX,interaction1='fath_rel',include=FALSE)
segregationeffects

# religion of father: include effect on eval
segregationeffects <- includeEffects(segregationeffects,sameX,interaction1='fath_rel',type='creation', include=TRUE)
segregationeffects

third.model <- sienaModelCreate(useStdInits=FALSE,projname='segregation.third',
                                cond=FALSE)

third.results <- siena07(third.model,data=segregationdata, 
                         effects=segregationeffects,batch=FALSE,verbose=FALSE, returnDeps=TRUE)
third.results
third.results <- siena07(third.model,data=segregationdata, 
                         effects=segregationeffects,batch=FALSE,verbose=FALSE,prevAns=third.results, returnDeps=TRUE)
third.results
# returnDeps=TRUE you need this for GOF evaluation


# If you look at the sequence of results, you see the effect size of 'same sex.M'
# drop, the more structural control parameters are included. When comparing
# the first and the third model, there even is a significant difference in model
# results: 95% confidence intervals for this parameter, which can be constructed
# with the help of the standard error estimates, do not overlap!


source("http://www.barkhof.uni-bremen.de/~mwindzio/sienaGOF.R")
gof.indegree4 <- sienaGOF(third.results, IndegreeDistribution, verbose=TRUE)
plot(gof.indegree4,key=0:8)
gof.outdegree4 <- sienaGOF(third.results,OutdegreeDistribution,verbose=TRUE)
plot(gof.outdegree4,key=0:8)
gof.geodesic4 <- sienaGOF(third.results,GeodesicDistribution,verbose=TRUE)
plot(gof.geodesic4,key=1:8)
gof.triads4 <- sienaGOF(third.results,TriadCensus,verbose=TRUE)
plot(gof.triads4, center=TRUE,scale=TRUE)

# key=triad.keys,

