## small_world.R

# example from Kolaczyk & Csardi (2014, p. 74p) 


library(igraph)
g.ws <- watts.strogatz.game(1, 25, 5, 0.05) # one grapgh, 25 vertices, 5 neighbours, 5% rewiring
plot(g.ws, layout=layout.circle, vertex.label=NA)
plot(g.ws, layout=layout.kamada.kawai, vertex.label=NA)

g.vert100 <- watts.strogatz.game(1, 100, 5, 0) # one grapgh, 100 vertices, 5 neighbours, 0% rewiring
plot(g.vert100, layout=layout.circle, vertex.label=NA)
transitivity(g.vert100)   # transitivity as indicator of clustering 
                          # high clustering

diameter(g.vert100)       # but the distances between the vertices are high  
                          # diameter is the longest geodesic in the network
                          # geodesic is the shortest path between two vertices (Prell 2012, p. 171)

average.path.length(g.vert100)
                          # again: the distances between the vertices are high

# no again the same with some rewiring
g.vert100_5p <- watts.strogatz.game(1, 100, 5, 0.05) # one grapgh, 100 vertices, 5 neighbours, 0% rewiring
plot(g.vert100_5p, layout=layout.circle, vertex.label=NA)
plot(g.vert100_5p, layout=layout.kamada.kawai, vertex.label=NA)
transitivity(g.vert100_5p)   # transitivity as indicator of clustering 
                             # still high clustering

diameter(g.vert100_5p)       # but the distances between the vertices are high  
                             # diameter is the longest geodesic in the network
                             # geodesic is the shortest path between two vertices (Prell 2012, p. 171)

average.path.length(g.vert100_5p)
                             # here, the distances between the vertices are >much< lower
#### Result: 5% rewiring dies not substantially affect clustering, but reduces path-lenght by 50% 


####
# now simulate small world network with 1000 nodes, 10 neighbours and varying p (=rewiring rate)
steps <- seq(-4, -0.5, 0.1) # sequence for the x axis, range and intervall size, 36 points
len <- length(steps)
cl <- numeric(len) # creates data set with 100 zeros
apl <- numeric(len)
ntrials <- 100
test <- numeric(ntrials) # creates data set with 100 zeros
for (i in (1:len)) {
  cltemp <- numeric(ntrials) #  creates data set with 100 zeros
  apltemp <- numeric(ntrials)
  for (j in (1:ntrials)) {
    g <-  watts.strogatz.game(1, 1000, 10, 10^steps[i]) # p changes in each, 10^-4=0.0001, 36 values
    cltemp[j] <- transitivity(g)
    apltemp[j] <- average.path.length(g)
  }
  cl[i] <- mean(cltemp)
  apl[i] <- mean(apltemp)
}
plot(steps, cl/max(cl))
plot(steps, cl/max(cl), ylim=c(0, 1), lwd=3, type="l", col="blue", xlab=expression(log[10](p)), ylab="Clustering and average path length") 
lines(steps, apl/max(apl), lwd=3, col="red")


